<?php

namespace Service\Event\Project;

use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\PostFlushEventArgs;
use Doctrine\ORM\Event\PreUpdateEventArgs;
use Doctrine\ORM\Events;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Log\ProjectLogInterface;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Subscriber implements EventSubscriber {
    
    protected $needsFlush;
    protected $logProject;
    
   	public function __construct(){
        $this->logProject = [];
    }

    /***********************************************************************
     * Check if is the entity an instance of ProjectLogInterface
     ***********************************************************************/
    private function isEntityProjectLogInterface($entity){
        return $entity instanceof ProjectLogInterface;
    }
    
    /***********************************************************************
     * Check if project status generates log.
     ***********************************************************************/
    private function isProjectStatusLogGenerator($projectStatus){
        return ($projectStatus && $projectStatus->getGeneratesLog() == 'Y');
    }
    
    public function preUpdate(PreUpdateEventArgs $e){
        $entity = $e->getEntity();
        $this->projectLog($e, $entity);
    }
    
    public function postUpdate(LifecycleEventArgs $e) {
		$entity = $e->getEntity();
        if(!empty($this->logProject) && $this->isEntityProjectLogInterface($entity)){
            /** @var \Service\ProjectLog\ProjectLog $serviceProjectLog */
            $serviceProjectLog = InstanceManager::getInstance()->getService('\Service\Log');
            $this->needsFlush = true;
            foreach ($this->logProject as $logProject){
                $serviceProjectLog->createLogUpdate($entity,$logProject['OLD_VALUE'],$logProject['NEW_VALUE'],$logProject['COLUMN']);
            }
            $this->logProject = array();
        }
	}
    
    /***********************************************************************
     * Mounts log value array
     ***********************************************************************/
    private function createLogStructure($oldValue, $newValue, $column){
        return array(
            'OLD_VALUE' => $oldValue,
            'NEW_VALUE' => $newValue,
            'COLUMN'    => $column
        );
    }
    /***********************************************************************
     * Add log to project's property
     ***********************************************************************/
    private function addProjectLog($oldValue, $newValue, $column){
        array_push($this->logProject, $this->createLogStructure($oldValue, $newValue, $column));

    }

    /**
     * @param PreUpdateEventArgs $e
     * @param ProjectLogInterface $entity
     */
     protected function projectLog(PreUpdateEventArgs $e, $entity){
        if($this->isEntityProjectLogInterface($entity)){

            if ($e->hasChangedField("projectStatus")){
                $oldValue = $e->getOldValue("projectStatus");
                $newValue = $entity->getProjectStatus();
                if ($this->isProjectStatusLogGenerator($oldValue) || $this->isProjectStatusLogGenerator($newValue)){
                    $newValue = $newValue ? $newValue->getId() : null;
                    $oldValue = $oldValue ? $oldValue->getId() : null;
                    $this->addProjectLog($oldValue, $newValue, 'PROJECT_STATUS');
                }
            }
        }
    }
    
    /***********************************************************************
     * Doctrine event. Called after a data is flushed in database.
     ***********************************************************************/
    public function postFlush(PostFlushEventArgs $e) {
        if($this->needsFlush) {
            $this->needsFlush = false;
            $e->getEntityManager()->flush();
        }
    }
    
    /***********************************************************************
	 * Gets registered events.
	 ***********************************************************************/
	public function getSubscribedEvents() {
		return array(
			Events::postUpdate,
			Events::preUpdate,
            Events::postFlush
		);
	}
}